use std::path::PathBuf;

use actix_web::{HttpRequest, HttpResponse};

pub struct ProxyMirror {
    mirrors: Vec<String>,
    data_dir: String,
    no_cache: regex::Regex,
}

impl ProxyMirror {
    pub fn new(mirrors: Vec<String>, data_dir: &str, no_cache: &str) -> Self {
        Self {
            mirrors,
            data_dir: data_dir.to_string(),
            no_cache: regex::Regex::new(no_cache).unwrap(),
        }
    }

    pub async fn get(&self, path: &str, req: &HttpRequest) -> Option<HttpResponse> {
        let p = std::path::Path::new(&path[1..]);
        let p = std::path::Path::new(&self.data_dir).join(p);

        // todo : fix file dir problem
        if !self.no_cache.is_match(path) {
            std::fs::create_dir_all(p.parent().unwrap()).unwrap();

            if p.exists() {
                // todo : refresh caches
                log::info!("Returning {path} from cache");
                return Some(
                    actix_files::NamedFile::open_async(&p)
                        .await
                        .ok()?
                        .into_response(req),
                );
            }
        }

        for mirror in &self.mirrors {
            log::info!("Fetching {path} from mirrors");
            let url = format!("{mirror}{path}");
            let res = self.get_url(&url, &p).await;
            if let Some(res) = res {
                if res.status().is_success() {
                    return Some(res);
                }
            }
        }
        None
    }

    pub async fn get_url(&self, path: &str, save: &PathBuf) -> Option<HttpResponse> {
        println!("Fetching {path}");
        let response = reqwest::get(path).await.unwrap();
        let status_code = response.status();
        let body_bytes = response.bytes().await.ok()?;
        std::fs::write(save, &body_bytes).unwrap();
        let mut http_response = HttpResponse::build(
            actix_web::http::StatusCode::from_u16(status_code.as_u16()).unwrap(),
        );
        let http_response = http_response.body(body_bytes);
        Some(http_response)
    }
}
